import 'package:flutter/material.dart';

enum AnimationType { normal, fadeIn }

/// Es el MainClass [context] es el BuildContext de donde venimos
/// [page] es el widget a donde queremos ir
/// [tipoAnimacion] es el tipo de animacion
/// [duracion] el tiempo que queremos que dure la animacion
/// [replacement] la bandera que sirve para saber si nos pasamos y borramos la pila de
/// widgets o solo nos pasamos a la que queremos sin borrar nada
class CustomFadeIn {
  final BuildContext context;
  final Widget page;
  final AnimationType tipoAnimacion;
  final Duration duracion;
  final bool replacement;

  /// Construtor del mainClass
  CustomFadeIn(
      {@required this.context,
      @required this.page,
      this.tipoAnimacion = AnimationType.normal,
      this.duracion = const Duration(milliseconds: 300),
      this.replacement = false}) {
    /// dependiendo del tipo de animacion haremos algo
    switch (this.tipoAnimacion) {
      case AnimationType.normal:
        this._navegacionNormal();
        break;
      case AnimationType.fadeIn:
        this._navegarFadeIn();
        break;
    }
  }

  /// Metodo que nos sirve para hacer un cambio de widget sencillo sin ningun tipo
  /// de animacion
  void _navegacionNormal() {
    //bien primero que nada vamos a crear la ruta
    final route = MaterialPageRoute(builder: (_) => this.page);

    /// condicinal que me sirve para saber si nos cambiamos con o sin mantener la pila
    /// de widgets
    if (this.replacement) {
      Navigator.pushReplacement(context, route);
    } else {
      Navigator.push(context, route);
    }
  }

  /// Metodo para hacer un cambio de widget con un efecto personalizado
  void _navegarFadeIn() {
    final route = PageRouteBuilder(
        pageBuilder: (_, __, ___) => this.page,
        transitionDuration: this.duracion,
        transitionsBuilder: (_, animation, __, child) {
          //bien entonces ahora si vamos a hacer la transicion
          return FadeTransition(
            child: child,
            opacity: Tween<double>(begin: 0.0, end: 1.0).animate(
                CurvedAnimation(parent: animation, curve: Curves.easeInOut)),
          );
        });

    /// Condicional para saber si mantenemos o no la pila de widgets
    if (this.replacement) {
      Navigator.pushReplacement(context, route);
    } else {
      Navigator.push(context, route);
    }
  }
}
